<?php
namespace Teknisa\Model\Gestaopessoas\Repositories;

class GpeQuadrovagas extends \Doctrine\ORM\EntityRepository {
    
    public function getQuadrosAtivos($nrOrg, $nrEstruturam, $nrOcupacao, $nrEscalatrabm = null): ?array {
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        
        $params = array(
            'P_NRORG'         => $nrOrg,
            'P_NRESTRUTURAM'  => $nrEstruturam,
            'P_NROCUPACAOM'   => $nrOcupacao,
            'P_NRESCALATRABM' => $nrEscalatrabm
        );

        $sql = <<<SQL
            SELECT QV.*
              FROM GPE_QUADROVAGAS QV
             WHERE QV.NRORG = :P_NRORG
                AND QV.NRESTRUTURAM = :P_NRESTRUTURAM
                AND QV.NROCUPACAOM = :P_NROCUPACAOM
                AND (QV.NRESCALATRABM = :P_NRESCALATRABM OR :P_NRESCALATRABM IS NULL)
                AND QV.IDATIVO = 'S'
                AND TRUNC(SYSDATE) BETWEEN NVL(QV.DTINIVALIDADE, TRUNC(SYSDATE)) AND NVL(QV.DTFIMVALIDADE, TRUNC(SYSDATE))
                AND QV.QTDLIMITE IS NOT NULL
            ORDER BY QV.DTINCLUSAO DESC
SQL;
        return $db->fetchAll($sql,$params);
    }
    
    public function getVinculosAtivosOcupEstrutHierarq($nrOrg, $dtMesCompetenc, $nrOcupacao, $nrEstrturam, $nrEscalatrabm = null): ?array {
        
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        
        $params = array(
            'P_NRORG'           => $nrOrg,
            'P_DTMESCOMPETENC'  => $dtMesCompetenc,
            'P_NRESTRUTURAM'    => $nrEstrturam,
            'P_NROCUPACAOM'     => $nrOcupacao,
            'P_NRESCALATRABM'   => $nrEscalatrabm
        );

        $sql = <<<SQL
            SELECT
                VM.*,
                VH.*
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
            )
            JOIN GPE_ALTEOCUPACAO AO ON AO.NRORG = VM.NRORG
                AND AO.NRVINCULOM = VM.NRVINCULOM
                AND AO.DTINIOCUPACAO = ( SELECT MAX(DTINIOCUPACAO) FROM GPE_ALTEOCUPACAO
                                          WHERE NRORG          = AO.NRORG
                                            AND NRVINCULOM     = AO.NRVINCULOM
                                            AND DTINIOCUPACAO <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')) )
            JOIN GPE_ALTEESCALA AL ON AL.NRORG = VM.NRORG
                AND AL.NRVINCULOM = VM.NRVINCULOM 
                AND AL.DTINIESCALA = ( SELECT MAX(DTINIESCALA) FROM GPE_ALTEESCALA
                                          WHERE NRORG          = AL.NRORG
                                            AND NRVINCULOM     = AL.NRVINCULOM
                                            AND DTINIESCALA <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')) )
            WHERE VM.NRORG = :P_NRORG
                AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
                AND (AO.NROCUPACAOM = :P_NROCUPACAOM OR :P_NROCUPACAOM IS NULL)
                AND (AL.NRESCALATRABM = :P_NRESCALATRABM OR :P_NRESCALATRABM IS NULL)
                AND EXISTS (
                    SELECT 1 FROM (
                        SELECT
                            HIER.NRESTRUTURAMREL
                        FROM HIERARQUIAORG HIER
                        WHERE HIER.NRORG = :P_NRORG
                        START WITH
                            HIER.NRESTRUTURAMREL = :P_NRESTRUTURAM
                        CONNECT BY NOCYCLE
                            PRIOR HIER.NRESTRUTURAMREL = HIER.NRESTRUTURAMSUP
                            AND PRIOR HIER.NRORG = HIER.NRORG
                    ) WHERE NRESTRUTURAMREL = VH.NRESTRUTGEREN
                )
SQL;
        return $db->fetchAll($sql,$params);
    }
    
    public function getQntVinculosAtivosOcupEstrutHierarq($nrOrg, $dtMesCompetenc, $nrOcupacao, $nrEstrturam, $nrEscalatrabm = null): int {
        return count($this->getVinculosAtivosOcupEstrutHierarq($nrOrg, $dtMesCompetenc, $nrOcupacao, $nrEstrturam, $nrEscalatrabm));
    }
    
    public function getEstruturasSuperiores($nrOrg, $nrEstrturam): ?array {
        
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        
        $params = array(
            'P_NRORG'           => $nrOrg,
            'P_NRESTRUTURAM'    => $nrEstrturam
        );

        $sql = <<<SQL
            SELECT
                NRORG,
                NIVEL,
                NRESTRUTURAM,
                NMESTRUTURAM,
                NRTIPOESTRUTURA,
                NMTIPOESTRUTURA,
                HIERARAQUIA
            FROM (
                SELECT
                    HIER.NRORG,
                    HIER.DTINCLUSAO,
                    HIER.NRESTRUTURAMSUP AS NRESTRUTURAM,
                    EM.NMESTRUTURAM,
                    EM.NRTIPOESTRUTURA,
                    TP.NMTIPOESTRUTURA,
                    LEVEL AS NIVEL,
                    SYS_CONNECT_BY_PATH(NMESTRUTURAM, ' <- ') HIERARAQUIA
                FROM HIERARQUIAORG HIER
                JOIN ESTRUTURAM    EM   ON EM.NRORG = HIER.NRORG AND EM.NRESTRUTURAM = HIER.NRESTRUTURAMSUP
                JOIN TIPOESTRUTURA TP ON (TP.NRORG = 0 OR TP.NRORG = HIER.NRORG) AND TP.NRTIPOESTRUTURA = EM.NRTIPOESTRUTURA
                WHERE HIER.NRORG = :P_NRORG
                START WITH
                    HIER.NRESTRUTURAMREL = :P_NRESTRUTURAM
                CONNECT BY NOCYCLE
                    PRIOR HIER.NRESTRUTURAMSUP = HIER.NRESTRUTURAMREL
                    AND PRIOR HIER.NRORG = HIER.NRORG
            )
            WHERE
                NRTIPOESTRUTURA = (
                    SELECT NRTPESTRUTQVAGAS FROM GPE_PARAMORG WHERE NRORG = :P_NRORG
                )
            ORDER BY NIVEL ASC, DTINCLUSAO DESC
SQL;
        return $db->fetchAll($sql,$params);
    }
    
}